@@ -7,8 +7,8 @@
 #' @export
 #'
 #' @examples
-#' fisher_r2z(0.5)
-fisher_r2z <- function(r){
+#' fisher_r_to_z(0.5)
+fisher_r_to_z <- function(r){
   z <- fisher_z(r)
   return(z)
 }
