@@ -27,6 +27,9 @@
 #'
 #' @param iter Number of iterations (posterior samples; defaults to 5000).
 #'
+#' @param impute Logicial. Should the missing values (\code{NA})
+#'               be imputed during model fitting (defaults to \code{TRUE}) ?
+#'
 #' @param analytic Logical. Should the analytic solution be computed (default is \code{FALSE})? This is only available
 #'                 for continous data. Note that if \code{type = "mixed"} and \code{analytic = TRUE}, the data will
 #'                 automatically be treated as continuous.
@@ -98,11 +101,18 @@
 #'
 #' }
 #'
+#' \strong{Imputing Missing Values}:
+#'
+#' Missing values are imputed with the approach described in \insertCite{hoff2009first;textual}{BGGM}.
+#' The basic idea is to impute the missing values with the respective posterior pedictive distribution,
+#' given the observed data, as the model is being estimated. Note that the default is \code{TRUE},
+#' but this ignored when there are no missing values. If set to \code{FALSE}, and there are missing
+#' values, list-wise deletion is performed with \code{na.omit}.
+#'
 #'
 #' @note
 #'
-#'
-#' \strong{Mixed Data:}
+#' \strong{Mixed Data}:
 #'
 #' The mixed data approach was introduced  \insertCite{@in @hoff2007extending;textual}{BGGM}
 #' (our paper describing an extension to Bayesian hypothesis testing if forthcoming).
@@ -187,8 +197,16 @@ ggm_compare_estimate <- function(...,
                                  analytic = FALSE,
                                  prior_sd = 0.50,
                                  iter = 5000,
+                                 impute = TRUE,
                                  progress = TRUE,
                                  seed = 1){
+
+  # temporary warning until missing data is fully implemented
+  if(type != "continuous"){
+    warning(paste0("imputation during model fitting is\n",
+                   "currently only implemented for 'continuous' data."))
+  }
+
   # combine data
   dat_list <- list(...)
 
@@ -224,6 +242,7 @@ ggm_compare_estimate <- function(...,
                mixed_type = mixed_type,
                seed = x,
                progress = progress,
+               impute = impute,
                ... = paste0("(Group ", x, ")"))
 
     })
